/**
 * @file
 * Fast Autocomplete jQuery plugin.
 */

(function ($) {
  var facJsonRequest = null;

  /**
   * Apply the fastAutocomplete plugin to given input:text elements.
   */
  $.fn.fastAutocomplete = function(options) {
    // Default settings.
    var settings = $.extend({
      id: null,
      jsonFilesPath: undefined,
      keyMinLength: 1,
      keyMaxLength: 5,
      breakpoint: 0,
      enabled: false,
      resizeTimer: null,
      emptyResult: '',
      allResultsLink: true,
      allResultsLinkThreshold: -1,
      highlightEnabled: false,
      resultLocation: false
    }, options);

    // Check if a jsonFilesPath is set before doing anything.
    if (settings.jsonFilesPath !== undefined) {
      // Only apply the plugin on input fields of type text.
      this.filter('input:text').each(function(index) {
        var that = this;

        $(this).attr({
          'autocomplete':'off',
          'autocorrect': 'off',
          'autocapitalize': 'none',
          'spellcheck': 'false',
          'aria-autocomplete':'list',
          'aria-owns': 'fac-result-' + settings.id + '-' + index
        });

        // Create the hidden result div.
        var facResult = $('<div>').attr({
          'class': 'fac-result',
          'id': 'fac-result-' + settings.id + '-' + index
        }).hide();

        toggleResponsiveBehavior(facResult, settings);
        $(window).resize(function(e) {
          clearTimeout(settings.resizeTimer);
          settings.resizeTimer = setTimeout(function() { toggleResponsiveBehavior(facResult, settings); }, 250);
        });

        // Add a hidden div to announce changes to screen readers.
        var announce = $('<div>').attr({
          'class': 'announce element-invisible',
          'id': 'fac-announce-' + settings.id + '-' + index,
          'role': 'status',
          'aria-live': 'polite'
        });
        announce.appendTo(facResult);

        // Create result list.
        var resultList = $('<ul>').attr({
          'class': 'result-list',
          'role' : 'listbox',
          'aria-labelledby' : 'fac-announce-' + settings.id + '-' + index
        }).appendTo(facResult).hide();

        if (settings.allResultsLink) {
          // Add the see all link to the result div.
          var seeAllLink = $('<li>').attr({
            'class': 'see-all-link',
            'id': 'fac-see-all-link-' + settings.id + '-' + index,
            'role': 'option'
          }).html('<div><a href="#">' + Drupal.t('See all results') + '</a></div></li>');
          // The mousedown and click events are to prevent the default link behavior.
          seeAllLink.find('> div > a').mousedown(function (e) {
            e.preventDefault();
          }).click(function (e) {
            e.preventDefault();
          });
          seeAllLink.css('cursor', 'pointer').mousedown(function (e) {
            switch (e.which) {
              // Left mouse click.
              case 1:
                $(that).closest('form').submit();
                break;
            }
            e.preventDefault();
          }).hover(function (e) {
            facResult.find('> ul.result-list > li.selected').removeClass('selected');
            $(this).addClass('selected');
            $(that).attr('aria-activedescendant', $(this).attr('id'));
          }).hide();
          seeAllLink.appendTo(resultList);
        }

        if (settings.emptyResult) {
          // Add the empty result text.
          var emptyResults = $('<div>').attr({
            'class': 'empty-result'
          });
          emptyResults.html(settings.emptyResult);
          emptyResults.find('a').each(function () {
            $(this).bind('mousedown', function (e) {
              e.preventDefault();
              switch (e.which) {
                // Left mouse click.
                case 1:
                  window.location = $(this).attr('href');
                  break;
              }
            });
          });
          emptyResults.appendTo(facResult);
        }

        var form = $(this).closest('form');
        if (settings.resultLocation) {
          facResult.appendTo(settings.resultLocation);
        }
        else {
          form.css('position', 'relative');
          facResult.appendTo(form);
        }
        // When a character is entered perform the necessary ajax call. Don't
        // respond to any special keys.
        $(this).bind('keyup', function(e) {
          if (settings.enabled) {
            if (!e) {
              e = window.event;
            }
            switch (e.keyCode) {
              case 9:
              case 16:
              case 17:
              case 18:
              case 20:
              case 33:
              case 34:
              case 35:
              case 36:
              case 37:
              case 38:
              case 39:
              case 40:
              case 13:
              case 27:
                return true;

              default:
                populateResults(this, facResult, settings);
                return true;
            }
          }
        });

        $(this).bind('paste', function(e) {
           $(that).trigger('keyup');
        });

        // Handle special keys (Enter, up, down).
        $(this).bind('keydown', function(e) {
          if (settings.enabled) {
            if (!e) {
              e = window.event;
            }
            switch (e.keyCode) {
              // Down arrow.
              case 40:
                selectDown(facResult, $(this));
                return false;

              // Up arrow.
              case 38:
                selectUp(facResult, $(this));
                return false;

              // Enter.
              case 13:
                var selected = facResult.find('li.selected:not(.see-all-link)');
                if (selected.length) {
                  window.location = selected.find('a:not(.contextual-links a)').attr('href');
                  return false;
                }
                else {
                  return true;
                }
                break;

              // Esc.
              case 27:
                if (facJsonRequest !== null) {
                  facJsonRequest.abort();
                }
                facResult.hide();
                return false;

              // All other keys.
              default:
                return true;

            }
          }
        });

        // Hide the result div when the input element loses focus.
        $(this).bind('blur', function(e) {
          if (settings.enabled) {
            if (facJsonRequest !== null) {
              facJsonRequest.abort();
            }
            facResult.hide();
          }
        });

        // When the input element gains focus, show the result.
        $(this).bind('focus', function(e) {
          if (settings.enabled) {
            facResult.show();
          }
        });
      });
    }

    // Return the original object to make the plugin chainable.
    return this;
  };

  // Enable or disable the Fast Autocomplete behavior based on a breakpoint.
  function toggleResponsiveBehavior(facResult, settings) {
    var browserWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
    if (browserWidth >= settings.breakpoint) {
      settings.enabled = true;
    }
    else {
      settings.enabled = false;
      if (facResult !== undefined) {
        facResult.hide();
      }
    }
  }

  // Populates the results div with suggestions from an AJAX response.
  function populateResults(element, facResult, settings) {
    var value = $(element).val();
    // Make sure the value is lowercase for case-insensitive search.
    value.toLowerCase();

    var resultList = facResult.find('> ul.result-list');
    var emptyResult = facResult.find('> div.empty-result');
    var announce = facResult.find('> div.announce');
    var seeAllLink = undefined;
    if (settings.allResultsLink) {
      seeAllLink = resultList.find('> li.see-all-link');
    }

    // Trim the result to the maximum length.
    if (value.length > settings.keyMaxLength) {
      value = value.substring(0, settings.keyMaxLength);
    }

    // Only perform the ajax call if the key has a minimum length. Append a
    // timestamp to the url to prevent the browser caching the json response.
    if (value.length >= settings.keyMinLength) {
      $(element).trigger('fac:requestStart');
      facJsonRequest = $.ajax({
        url: settings.jsonFilesPath + value + '.json?nocache=' + (new Date()).getTime(),
        dataType: 'json',
        type: 'GET',
        processData: false,
        beforeSend : function() {
          if (facJsonRequest !== null) {
            facJsonRequest.abort();
          }
        },
        success: function(data, status, xhr) {
          if (data.items.length) {
            announce.text(Drupal.formatPlural(data.items.length, '@count result found.', '@count results found.') + ' ' + Drupal.t('Use the up/down keys to navigate the results or press <ENTER> to show all results.'));
            if (seeAllLink) {
              if (data.items.length >= settings.allResultsLinkThreshold) {
                seeAllLink.find('> div > a').html(Drupal.t('See all results for "%key"', {'%key': $(element).val()}));
                seeAllLink.show();
              }
              else {
                seeAllLink.hide();
              }
            }
            resultList.find('> li.result').remove();
            emptyResult.hide();
            $.each(data.items, function(key, dataValue) {
              var item = $('<li class="result" role="option" id="facoption' + key + '">' + dataValue + '</li>');
              item.css('cursor', 'pointer').mousedown(function(e) {
                e.preventDefault();
                switch (e.which) {
                  // Left mouse click.
                  case 1:
                    /// Open in new window/tab if the metakey was pressed while clicking.
                    if (e.metaKey) {
                      window.open(item.find('a:not(.contextual-links a)').attr('href'));
                    }
                    else {
                      window.location = item.find('a:not(.contextual-links a)').attr('href');
                    }
                    break;
                }
              }).hover(function(e) {
                resultList.find('> li.selected').removeClass('selected');
                $(element).attr('aria-activedescendant', 'facoption' + key);
                $(this).addClass('selected');
              });
              if (seeAllLink !== undefined) {
                item.insertBefore(seeAllLink);
              }
              else {
                item.appendTo(resultList);
              }
            });
            resultList.show();
            facResult.show();
            if (settings.highlightEnabled && $.highlight) {
              resultList.find('> li.result').unhighlight();
              resultList.find('> li.result').highlight($(element).val().split(' '));
            }
          }
          else {
            announce.text('No results found.');
            resultList.hide();
            resultList.find('li.result').remove();
            if (seeAllLink) {
              seeAllLink.hide();
            }
            facResult.hide();
            emptyResult.show();
          }
          $(element).trigger('fac:requestEnd');
        },
        fail: function() {
          $(element).trigger('fac:requestEnd');
        }
      });
    }
    else {
      if (settings.highlightEnabled && $.highlight) {
        resultList.find('> li.result').unhighlight();
        resultList.find('> li.result').highlight($(element).val().split(' '));
      }
      // If the key is empty, clear the result div and show the empty result content.
      if (value.length < 1) {
        announce.text('');
        resultList.hide();
        resultList.find('> li.result').remove();
        if (seeAllLink) {
          seeAllLink.hide();
        }
        facResult.hide();
        emptyResult.show();
      }
    }
  }

  // Select the next suggestion.
  function selectDown(facResult, input_element) {
    var selector = '> div.empty-result ul';
    if (facResult.find('> ul.result-list > li').length) {
      selector = '> ul.result-list';
    }
    var selected = facResult.find(selector + ' > li.selected');
    if (selected.length) {
      selected.removeClass('selected');
      selected.next('li').addClass('selected');
      input_element.attr('aria-activedescendant', selected.next('li').attr('id'));
    }
    else {
      facResult.find(selector + ' > li:first').addClass('selected');
      input_element.attr('aria-activedescendant', facResult.find(selector + ' > li:first').attr('id'));
    }
  }

  // Select the previous suggestion.
  function selectUp(facResult, input_element) {
    var selector = '> div.empty-result ul';
    if (facResult.find('> ul.result-list > li').length) {
      selector = '> ul.result-list';
    }
    var selected = facResult.find(selector + ' > li.selected');
    if (selected.length) {
      selected.removeClass('selected');
      selected.prev('li').addClass('selected');
      input_element.attr('aria-activedescendant', selected.prev('li').attr('id'));
    }
    else {
      facResult.find(selector + ' > li:last').addClass('selected');
      input_element.attr('aria-activedescendant', facResult.find(selector + ' > li:last').attr('id'));
    }
  }

}(jQuery));
